#!/usr/bin/env python
# -*- coding: UTF-8 -*-

import sys
import os
import re
import time
import json
import socket
import logging
import threading
import traceback
from qmf.console import Session
from thrift.transport import TSocket
from thrift.transport import TTransport
from thrift.protocol import TBinaryProtocol
from thrift.server import TServer

from snakemq.queues import QueuesManager
from snakemq.message import Message, FLAG_PERSISTENT
from snakemq.storage.sqlite import SqliteQueuesStorage

import logger
import propertyReader
from plugins import status
from syncThrift import CashCommunicationService
from syncThrift.ttypes import *


loggerSync =  logging.getLogger('su.artix.loggerSync')
lockLoad = threading.Lock()

def unloadTasksHandle():
    queueTasks = None
    while True:
        nextSleep = 3
        try:
            queueStorage = SqliteQueuesStorage("unloadTasks.db")
            queueManager = QueuesManager(queueStorage)
            queueTasks = queueManager.get_queue('unloadTasks')
            for i in range(len(queueTasks)):
                taskStr = queueTasks.get().data
                if taskStr != None:
                    task = json.loads(taskStr, encoding='utf8')
                    res = unloadRun(task['fromDate'], task['toDate'], task['shopId'], task['cashCode'], task.get('configInfo'), task['saleStateId'])
                    if res == status.busy:
                        nextSleep = 3
                        break
                    elif res == status.OK:
                        queueTasks.pop()
                    else:
                        nextSleep = 60
                        break
        except Exception, e:
            nextSleep = 60
            loggerSync.error('Произошла ошибка во время обработки задач на выгрузку. Попробуем обработать через минуту.')
            loggerSync.error(traceback.format_exc())
        finally:
            if queueTasks and queueTasks.manager:
                queueTasks.manager.close()
        time.sleep(nextSleep)

def unloadRun(fromDate, toDate, shopId, cashCode, configInfo, saleStateId):
    method = 'unload'
    try:
        reconfigure(configInfo)
        status.sendStatus(loggerSync, method, status.start, 'Start unload')
        status.updateSaleState(saleStateId, 'Start unload', 'Начинаем выгрузку продаж', status.STATUS_DURING)    
    
        loggerSync.info('Начинаем выгрузку продаж')
        plugin = __import__('unloadRun', fromlist='q')
        rez, messEn, messRu = plugin.main(propertyReader.localBroker, propertyReader.cashCode, propertyReader.storeID, propertyReader.mode, 
                                          dates = '%s@%s' % (fromDate, toDate), saleStateId = saleStateId, unloadingShopId=shopId, unloadingCashCode=cashCode)
        loggerSync.info('Вернулись из плагина')
        status.sendStatus(loggerSync, method, rez, messRu)
        loggerSync.info('Отправили статус на сервер')
        status.updateSaleState(saleStateId, messEn, messRu, status.STATUS_READY if rez == status.OK else status.STATUS_ERROR)
        loggerSync.info('Обновили статус выгрузки продаж на сервере')
        return rez
    except Exception, e:
        loggerSync.error('Error 0, '+str(e))
        status.sendStatus(loggerSync, method, status.errorProgramm, str(e))
        status.updateSaleState(saleStateId, str(e), str(e), status.STATUS_ERROR)
    return status.errorUnload

def removeRoutesByQueue(broker, queue, remoteBroker, routeDestExchange):
    sess = Session()
    _connTimeout = 10
    localBroker = sess.addBroker(broker, _connTimeout)
    localBroker._waitForStable()
    localBroker.getAmqpSession().queue_declare(queue=queue, passive=False, durable=True, arguments={'qpid.file_size': 24, 'qpid.file_count': 8})
    bridges = sess.getObjects(_class="bridge", dynamic=False)
    allLinks = []
    for bridge in bridges:
        allLinks.append(bridge._linkRef_)
    changedRemoteBroker = True
    for bridge in bridges:
        if bridge.src == queue:
            remotelink = bridge._linkRef_
            if remoteBroker.split(':')[0] == remotelink.host and remoteBroker.split(':')[1] == str(remotelink.port) and bridge.dest == routeDestExchange:
                changedRemoteBroker = False
                loggerSync.info('Routing already added')
                continue

            for i in range(len(allLinks)):
                if allLinks[i].host == remotelink.host and allLinks[i].port == remotelink.port:
                    allLinks.__delitem__(i)
                    break
            loggerSync.info("Removing route (from broker=%s queue=%s to broker=%s:%s exchange=%s)" % (bridge.getBroker().getUrl(), bridge.src, remotelink.host, remotelink.port, bridge.dest))
            res = bridge.close()
            if res.status != 0:
                loggerSync.error("Error closing bridge: %d - %s" % (res.status, res.text))

            delLinkFlag = True
            for i in range(len(allLinks)):
                if allLinks[i].host == remotelink.host and allLinks[i].port == remotelink.port:
                    delLinkFlag = False
                    break
            if delLinkFlag:
                loggerSync.info("Closing link %s:%s" % (remotelink.host, remotelink.port))
                res = remotelink.close()
                if res.status != 0:
                    loggerSync.error("Error closing link: %d - %s" % (res.status, res.text))

    try:
        sess.delBroker(localBroker)
        sess.close()
    except Exception, e:
        loggerSync.error('Error on broker session closing. %s' % str(e))
    return changedRemoteBroker

class CashCommunicationHandler:
  #def __init__(self):
#    self.log = {}
#    self.loadWork = False
#    self.unloadWork = False

    def load(self, loadScript, configInfo=None):
        method = 'load'
        if lockLoad.acquire(False):
            try:
                res = status.errorProgramm
                reconfigure(configInfo.__dict__ if configInfo else None,)
                status.sendStatus(loggerSync, method, status.start, 'Start load')
                loggerSync.info('Начинаем загружать справочники')

                loggerSync.info("Идентификатор потока, захватившего загрузку %s, %s" % (str(threading.current_thread().ident), threading.current_thread().name))
                plugin = __import__('sync', fromlist='q')
                rez, mess = plugin.main(propertyReader.remoteBroker, propertyReader.cashCode, propertyReader.brokerConnectionHeartbeat)
                loggerSync.info('Вернулись из плагина')
                status.sendStatus(loggerSync, method, rez, mess)
                loggerSync.info('Отправили статус на сервер')
            except Exception, e:
                loggerSync.error('Error 0, '+str(e))
                status.sendStatus(loggerSync, method, status.errorProgramm, str(e))
                res = status.errorProgramm
            finally:
                loggerSync.info('Зашли в finally')
                loggerSync.info("Идентификатор потока, разблокировавший загрузку %s, %s" % (str(threading.current_thread().ident), threading.current_thread().name))
                lockLoad.release()
                loggerSync.info('Процесс загрузки разблокирован')
                return res
        else:
            loggerSync.info('Процесс загрузки справочников занят!')
            loggerSync.info("Отказано потоку с идентификатором %s, %s" % (str(threading.current_thread().ident), threading.current_thread().name))
            status.sendStatus(loggerSync, method, status.busy, 'Busy')
            return status.busy

    def unload(self, unloadScript, fromDate, toDate, shopId=None, cashCode=None, configInfo=None):
        method = 'unload'
        try:
            saleStateId = None
            addToQueue = True
            queueStorage = SqliteQueuesStorage("unloadTasks.db")
            queueManager = QueuesManager(queueStorage)
            queueTasks = queueManager.get_queue('unloadTasks')
            
            # если в очереди уже есть запросы с диапазоном дат входящим в диапазон текущего запроса, то изменим начало текущего запроса
            try:
                for i in range(len(queueTasks)):
                    taskStr = queueTasks.queue[i].data
                    if taskStr != None:
                        taskFromQueue = json.loads(taskStr, encoding='utf8')
                        if taskFromQueue['shopId'] == shopId and taskFromQueue['cashCode'] == cashCode:
                            if taskFromQueue['fromDate'] == fromDate and taskFromQueue['toDate'] == toDate:
                                messRu = u'Задача за диапазон [%s - %s] уже поставлена в очередь на обработку' % (fromDate, toDate)
                                loggerSync.info(messRu)
                                addToQueue = False
                                saleStateId = status.addSaleState('Task for date [%s - %s] already in queue' % (fromDate, toDate), messRu, status.STATUS_READY, fromDate, toDate, callFromCS = True)
                            elif taskFromQueue['fromDate'] <= fromDate and taskFromQueue['toDate'] > fromDate and taskFromQueue['toDate'] < toDate:
                                loggerSync.info(u'В очереди уже есть запрос на выгрузку продаж, диапазон дат которого перекрывает часть текущего запроса. Изменяем время начала запроса продаж %s -> %s' % (fromDate, taskFromQueue['toDate']))
                                fromDate = taskFromQueue['toDate']
            except Exception, e:
                loggerSync.warning(u'Не удалось проверить необработанные запросы продаж. %s' % e)
                loggerSync.error(traceback.format_exc())
                
            if addToQueue:
                saleStateId = status.addSaleState('Putting task for unload sales to queue', 'Ставим в очередь задачу на выгрузку продаж', status.STATUS_DURING, fromDate, toDate, callFromCS = True)
                task = {'fromDate': fromDate, 'toDate': toDate, 'shopId': shopId, 'cashCode': cashCode, 'configInfo': configInfo.__dict__ if configInfo else None,'saleStateId': saleStateId}
                queueTasks.push(Message(json.dumps(task), ttl=60, flags=FLAG_PERSISTENT))
                loggerSync.info(u'Задача на выгрузку продаж поставлена в очередь (fromDate=%s, toDate=%s, shopId=%s, cashCode=%s).' % (fromDate, toDate, shopId, cashCode))
            queueTasks.manager.close()
            
            res = status.OK
        except Exception, e:
            loggerSync.error('Error 0, '+str(e))
            status.sendStatus(loggerSync, method, status.errorProgramm, str(e))
            status.updateSaleState(saleStateId, str(e), str(e), status.STATUS_ERROR)
            res = status.errorProgramm
        finally:
            loggerSync.info('Зашли в finally')
            loggerSync.info('Процесс выгрузки разблокирован')
            return res

def reconfigure(configInfo):
    if not configInfo is None:
        for k, v in configInfo.items():
            configInfo[k] = v.encode('utf8') if isinstance(v, unicode) else v
        changedConfigProperties = False
        changedQpidProperties = False
        cashId = configInfo.get('cashId')
        if not cashId is None and propertyReader.cashCode != cashId:
            loggerSync.info(u'Изменяем настройку cash.code: %s -> %s' % (propertyReader.cashCode, cashId))
            propertyReader.cashCode = cashId
            rewriteConfig('cash.code', cashId)
            changedConfigProperties = True

        shopId = configInfo.get('shopId')
        if not shopId is None and propertyReader.storeID != shopId:
            loggerSync.info(u'Изменяем настройку storeID: %s -> %s' % (propertyReader.storeID, shopId))
            propertyReader.storeID = shopId
            rewriteConfig('storeID', shopId)
            changedConfigProperties = True

        qpidHostCS = configInfo.get('qpidHostCS')
        httpHostCS = configInfo.get('httpHostCS')
        if not qpidHostCS is None: 
            if propertyReader.remoteIp != qpidHostCS:
                loggerSync.info(u'Изменяем настройку dept.broker.ip: %s -> %s' % (propertyReader.remoteIp, qpidHostCS))
                propertyReader.remoteIp = qpidHostCS
                propertyReader.remoteBroker = re.sub(r"@(.+):", "@%s:" % qpidHostCS, propertyReader.remoteBroker)
                rewriteConfig('dept.broker.ip', qpidHostCS)
                changedQpidProperties = True
                changedConfigProperties = True
        elif not httpHostCS is None and propertyReader.remoteIp != httpHostCS:
            loggerSync.info(u'Изменяем настройку dept.broker.ip: %s -> %s' % (propertyReader.remoteIp, httpHostCS))
            propertyReader.remoteIp = httpHostCS
            propertyReader.remoteBroker = re.sub(r"@(.+):", "@%s:" % httpHostCS, propertyReader.remoteBroker)
            rewriteConfig('dept.broker.ip', httpHostCS)
            changedQpidProperties = True
            changedConfigProperties = True

        qpidPortCS = configInfo.get('qpidPortCS')
        if (not qpidPortCS is None) and (qpidPortCS != 0) and (int(propertyReader.remotePort) != int(qpidPortCS)):
            loggerSync.info(u'Изменяем настройку dept.broker.port: %s -> %s' % (propertyReader.remotePort, qpidPortCS))
            propertyReader.remotePort = qpidPortCS
            propertyReader.remoteBroker = re.sub(r":\d+", ":%s" % qpidPortCS, propertyReader.remoteBroker)
            rewriteConfig('dept.broker.port', qpidPortCS)
            changedQpidProperties = True
            changedConfigProperties = True

        httpPortCS = configInfo.get('httpPortCS')
        if (not httpPortCS is None) and (httpPortCS != 0) and (int(propertyReader.cashServerRestPort) != int(httpPortCS)):
            loggerSync.info(u'Изменяем настройку cashServerRestPort: %s -> %s' % (propertyReader.cashServerRestPort, httpPortCS))
            propertyReader.cashServerRestPort = httpPortCS
            rewriteConfig('cashServerRestPort', httpPortCS)
            changedConfigProperties = True

        dictUrlHostCS = configInfo.get('dictUrlHostCS')
        if (not dictUrlHostCS is None) and (propertyReader.dictUrlHostCS != dictUrlHostCS):
            loggerSync.info(u'Изменяем настройку dictUrlHostCS: %s -> %s' % (propertyReader.dictUrlHostCS, dictUrlHostCS))
            propertyReader.dictUrlHostCS = dictUrlHostCS
            rewriteConfig('dictUrlHostCS', dictUrlHostCS)
            changedConfigProperties = True

        dictUrlPortCS = configInfo.get('dictUrlPortCS')
        if (not dictUrlPortCS is None) and (propertyReader.dictUrlPortCS != dictUrlPortCS):
            loggerSync.info(u'Изменяем настройку dictUrlPortCS: %s -> %s' % (propertyReader.dictUrlPortCS, dictUrlPortCS))
            propertyReader.dictUrlPortCS = dictUrlPortCS
            rewriteConfig('dictUrlPortCS', dictUrlPortCS)
            changedConfigProperties = True

        if changedConfigProperties:
            loggerSync.info(u'Перезаписываем конфикурационный файл.')
            with open(propertyReader.pathProperties, 'wb') as configfile:
                propertyReader.config.write(configfile)

        if changedQpidProperties:
            configureRouting()

def rewriteConfig(propertyName, value):
    propertyReader.config.set('Properties', propertyName, value.encode('utf8') if isinstance(value, unicode) else value)

def startService(transport):
        handler = CashCommunicationHandler()
        processor = CashCommunicationService.Processor(handler)
        tfactory = TTransport.TBufferedTransportFactory()
        pfactory = TBinaryProtocol.TBinaryProtocolFactory()
        
        #server = TServer.TSimpleServer(processor, transport, tfactory, pfactory)
        
        # You could do one of these for a multithreaded server
        server = TServer.TThreadedServer(processor, transport, tfactory, pfactory)
        #server = TServer.TThreadPoolServer(processor, transport, tfactory, pfactory)
        
        loggerSync.info('Starting the server...')
        server.serve()
        loggerSync.info('Done.')

def createAndBindQueueToExchane(broker, queue, exchange=None, routingKey='#'):
    if exchange:
        sess = Session()
        _connTimeout = 10
        localBroker = sess.addBroker(broker, _connTimeout)
        localBroker._waitForStable()
        localBroker.getAmqpSession().queue_declare(queue=queue, passive=False, durable=True, arguments={'qpid.file_size': 24, 'qpid.file_count': 8})
    
        localBroker.getAmqpSession().exchange_declare(exchange=exchange, passive=False, durable=True, type='topic', auto_delete=False)
        localBroker.getAmqpSession().exchange_bind(exchange=exchange, queue=queue, binding_key=routingKey)
        try:
            sess.delBroker(localBroker)
            sess.close()
        except Exception, e:
            loggerSync.error('Error on broker session closing. %s' % str(e))

def configureRouting():
    routeQueue = 'artix.backward.cs'
    bindExchange = routeDestExchange = 'artix.backward.cs'
    try:
        createAndBindQueueToExchane(broker = propertyReader.localBroker, queue=routeQueue, exchange=bindExchange)
        if (propertyReader.mode == 'server'):
            routeQueue = 'artix.backward.mcs'
            createAndBindQueueToExchane(broker = propertyReader.localBroker, queue=routeQueue, exchange=bindExchange)
        addNewRoute = removeRoutesByQueue(broker = propertyReader.localBroker, queue = routeQueue, remoteBroker=propertyReader.remoteBroker.split('@')[1], routeDestExchange = routeDestExchange)
        if addNewRoute:
            loggerSync.info("Adding route from broker=%s queue=%s to broker=%s exchange=%s" % (propertyReader.localBroker.split('@')[1], routeQueue, propertyReader.remoteBroker.split('@')[1], routeDestExchange))
            os.system("qpid-route -d -s queue add %s %s %s %s" % (propertyReader.remoteBroker, propertyReader.localBroker, routeDestExchange, routeQueue))
    except socket.error, e:
        message = 'Ошибка подключения к локальному сервису qpid. '
        loggerSync.error(message + str(e))

#
#   Данный метод необходимо выполнить здесь, так как 
#   в exchangers в модуле startReader, который используется для формирования
#   выгрузки продаж, используется модуль signal, который должен быть подгружен
#   только в главном потоке. Иначе будет ошибка импортирования.
#   Такой же импорт находится в модуле artixinterchange.py
#
def importUnloadModule():
    try:
        curPath = sys.path[0]
        unloadScript = propertyReader.unloadScript
        sys.path[0] = '%s' % os.path.dirname(unloadScript)
        __import__(os.path.basename(os.path.splitext(unloadScript)[0]), fromlist='q')
        sys.path[0] = curPath
    except Exception, e:
        messRu = 'Ошибка при подключении модуля выгрузки продаж.'
        loggerSync.error('%s %s' % (messRu, str(e)))
        os._exit(1)


if __name__ == '__main__':
    importUnloadModule()
    configureRouting()
    transport = TSocket.TServerSocket('0.0.0.0', 7795)
    unloadTasksHandler = threading.Thread(target=unloadTasksHandle)
    unloadTasksHandler.daemon = True
    unloadTasksHandler.start()
    startService(transport)
