# -*- coding: UTF-8 -*-

import os, sys
import zipfile
import glob
from datetime import timedelta, datetime
import logging
from operator import itemgetter
import propertyReader
log = logging.getLogger('su.artix.loggerSync')

def sort_files_by_last_modified(files):
    fileData = {}
    for fname in files:
        fileData[fname] = os.stat(fname).st_mtime
    fileData = sorted(fileData.items(), key = itemgetter(1))
    return fileData

def archive(srcFile, directory, maxCountFiles=propertyReader.maxCountBackupFiles, actualCountDays=propertyReader.actualCountDaysForBackupFiles):
    dstDir = '%s/../backup/%s' % (sys.path[0], directory)
    if not os.path.exists(dstDir):
        os.makedirs(dstDir)
    if os.path.exists(srcFile):
        log.info('Переносим в архив файл %s' % srcFile)
        fileName = os.path.basename(os.path.splitext(srcFile)[0])
        root = dstDir + '/' + fileName
        ext='.zip'
        vers = ''
        num = 1
        while os.path.exists(root+vers+ext):
            vers = '_' + str(num)
            num+=1
        backup = root +vers + ext
        gz=zipfile.ZipFile (backup, 'a', 8)
        gz.write(srcFile, None, 8)
        os.remove(srcFile)
        gz.close()
        log.info('Создан архив %s' % backup)
    else:
        log.warning('Файл %s не найден.' % srcFile)

    if maxCountFiles > 0:
        files = glob.glob('%s/*.zip' % dstDir)
        sorted_files = sort_files_by_last_modified(files)
        deleteCount = len(sorted_files) - maxCountFiles
        for x in range(0, deleteCount):
            log.info("Удаляем устаревший архив %s" % sorted_files[x][0])
            os.remove(sorted_files[x][0])
    if actualCountDays > 0:
        files = glob.glob('%s/*.zip' % dstDir)
        for fname in files:
            actualDate = datetime.now() - timedelta(days=actualCountDays)
            if datetime.fromtimestamp(os.stat(fname).st_mtime) < actualDate:
                log.info("Удаляем устаревший архив %s" % fname)
                os.remove(fname)

