# -*- coding: UTF-8 -*-

import os
import logging
import status
import tempfile
import backupFile
import bz2
import propertyReader

log = logging.getLogger('su.artix.loggerSync')
#
# Получение справочников из очереди
#

path = tempfile.gettempdir()
fileName = 'pos1'

# ключи заголовка сообщения
dictLocationKey = 'dictLocation'
formatKey = 'format'

def loadDict(Format, dictPath, messageId, dictCashLoadStateId = None, version = None):
    dictPath = os.path.normpath(dictPath)
    try:
        exec 'import ' + Format.lower() + ' as module'
    except:
        mes = 'Не найден модуль обработки справочника формата %s' % Format.encode('utf8')
        log.error(mes)
        status.sendDictState(dictCashLoadStateId, 'Not found module %s' % Format.encode('utf8'), mes, status.STATUS_ERROR, version)
        return status.errorProgramm, mes
    try:
        res, mes = module.upload(dictPath, messageId, dictCashLoadStateId, version)
        backupFile.archive(dictPath, 'dicts')
        return res, mes
    except Exception, e:
        log.error(str(e))
        status.sendDictState(dictCashLoadStateId, str(e), 'Ошибка: %s' % str(e), status.STATUS_ERROR, version)
        return status.errorProgramm, str(e)

def getActualFileName(fName):
    vers = ''
    num = 1
    while os.path.exists(path+'/'+fName+vers):
        vers = '_' + str(num)
        num+=1
    fName = fName + vers
    return fName

def main(message):
    fileName = 'pos.aif'
    messageId = message.id
    log.info('Определяем формат передаваемого справочника')
    format = message.properties.get(formatKey)
    version = message.properties.get('version')
    compressMethod = message.properties.get('compressMethod')
    dictMessageType = message.properties.get("dictMessageType", "FULL").upper()
    dictCashLoadStateId = message.properties.get('dictCashLoadStateId' + propertyReader.cashCode)
    if version >= '3':
        dictCashLoadStateId = message.properties.get('dictGenerateStateId')
    status.sendDictState(dictCashLoadStateId, 'Get dict from queue', 'Получаем справочник из очереди', status.STATUS_DURING, version)
    if format == None:
        mes = 'Формат справочника не указан.'
        log.warning(mes)
        return status.errorLoadDict, mes
    else:
        log.info('Формат справочника - %s.' % format.encode('utf8'))

        log.info('Определяем способ передачи справочника.')
        dictLocation = message.properties.get(dictLocationKey)
    
        if dictLocation == 'body':
            log.info('Передача справочника через тело сообщения. Получаем тело сообщения.')
            body = message.content
            if body == None:
                mes = 'Cправочник не указан в теле сообщения.'
                log.warning(mes)
                return status.errorLoadDict, mes
            else:
                import writetofile
                
                openModeFileWrite = 'ab' if dictMessageType in ['MIDDLE_PART', 'LAST_PART'] else 'wb'
                writeFileName = os.path.splitext(fileName)[0]+'.bz2' if compressMethod == 'BZ2' else fileName
                writetofile.main(body, path, writeFileName, openModeFileWrite)
                
                if dictMessageType in ['MIDDLE_PART', 'FIRST_PART']:
                    mes = 'Пойдем в очередь за следующей частью справочника'
                    log.info(mes)
                    return status.OK, mes
                
                if compressMethod == 'BZ2':
                    decompressFile(os.path.join(path, writeFileName), os.path.join(path,fileName))
                
                # Загрузка справочника в указанном формате
                res, mes = loadDict(format, os.path.join(path, fileName), messageId, dictCashLoadStateId, version)
                return res, mes

        elif dictLocation == 'url':
            log.info('Передача через URL. Получаем URL справочника.')
            url = message.content
            if url == None:
                mes = 'Не указана ссылка на скачивание справочника.'
                log.warning(mes)
                return status.errorLoadDict, mes
            else:
                url = url.replace('{host}', str(propertyReader.dictUrlHostCS)).replace('{port}', str(propertyReader.dictUrlPortCS))
                log.info('URL - %s' % url)
                import downloadfile
                fileName = url.split('/')[len(url.split('/'))-1]
                fileName = getActualFileName(fileName)
                res, mes = downloadfile.main(url, path, fileName)
                if res and compressMethod == 'BZ2':
                    newFileName = fileName.replace('.bz2', '')
                    decompressFile(os.path.join(path, fileName), os.path.join(path, newFileName))
                    fileName = newFileName

                # Загрузка справочника в указанном формате
                if res:
                    res, mes = loadDict(format, os.path.join(path, fileName), messageId, dictCashLoadStateId, version)
                else:
                    res = status.errorLoadDict
                return res, mes
        else:
            mes = 'Способ передачи справочника не определен.'
            log.warning(mes)
            return status.errorLoadDict, mes

def decompressFile(sourceFile, destFile):
    f = open(destFile, 'w')
    data = bz2.BZ2File(sourceFile, 'rb')
    chunk = data.read(1048576) # считываем 1 МБ
    while chunk:
        f.write(chunk)
        chunk = data.read(1048576) # считываем 1 Мб
    data.close()
    f.close()
    os.remove(sourceFile)

