# -*- coding: UTF-8 -*-

import sys
import os
import ConfigParser
import logging

loggerSync =  logging.getLogger('su.artix.loggerSync')

def getValue(option, defaultValue):
    '''
    Получить значение опции из конфигурационного файла.
    В случае ошибки вернуть значение по умолчанию
    '''
    value = None
    try:
        if type(defaultValue) is int:
            value = config.getint('Properties', option)
        elif type(defaultValue) is bool:
            value = config.getboolean('Properties', option)
        else:
            value = config.get('Properties', option)
    except ConfigParser.NoOptionError:
        loggerSync.warning(u'Не найдена настройка %s. Берем значение по умолчанию: %s' % (option, defaultValue))
        value = defaultValue

    loggerSync.info(u'%s - %s' % (option.ljust(40, ' '), str(value).ljust(20, ' ')))
    return value

def getRequiredValue(option):
    '''
    Получить обязательное значение из конф-го файла
    В случае, если нет секции или опции, генерируется Exception
    '''
    value = config.get('Properties', option)
    loggerSync.info(u'%s - %s' % (option.ljust(40, ' '), str(value).ljust(20, ' ')))
    return value

# Считывание настроек подключения sync-агента
pathProperties = os.path.normpath('%s/../properties/cash.ini' % sys.path[0])
if os.access(pathProperties, os.F_OK):
    config = ConfigParser.RawConfigParser()
    config.read(pathProperties)
    localBroker = 'guest/guest@127.0.0.1:5672'
    remotePort = getValue('dept.broker.port', 5672)
    maxCountBackupFiles = getValue('maxCountBackupFiles', 0)
    actualCountDaysForBackupFiles = getValue('actualCountDaysForBackupFiles', 0)
    cashServerRestPort = getValue('cashServerRestPort', 8080)
    queueFileSize = getValue('queueFileSize', 128)
    queueFileCount = getValue('queueFileCount', 16)
    queueMaxSize = getValue('queueMaxSize', 104857600)
    queueMaxCount = getValue('queueMaxCount', 3000)
    mode = getValue('mode', 'cash')
    uploadScript = getValue('uploadScript', '/linuxcash/cash/exchangesystems/exchangers/src/upload/converter/aif.py')
    useQueueInUpload = getValue('useQueueInUpload', True)
    brokerConnectionHeartbeat = getValue('brokerConnectionHeartbeat', 0)
    dictUrlHostCS = getValue('dictUrlHostCS', '127.0.0.1')
    dictUrlPortCS = getValue('dictUrlPortCS', 80)

    try:
        # Значения, которые обязательно должны присутствовать в конф-ом файле
        remoteIp = getRequiredValue('dept.broker.ip')
        unloadScript = getRequiredValue('unloadScript')
        cashCode = getRequiredValue('cash.code')
        storeID = getRequiredValue('storeID')
        remoteBroker = 'guest/guest@%s:%s' % (remoteIp, remotePort)
    except Exception, e:
        loggerSync.error(str(e))
        sys.exit()

else:
    loggerSync.error(u'Нет файла с настройками')
    sys.exit()