# -*- coding: UTF-8 -*-

import os, sys
import re
from qpid.messaging import *
import time
from datetime import datetime
import logging
import zipfile
import logger
loggerSync =  logging.getLogger('su.artix.loggerSync')
from plugins import status, backupFile
import tempfile
import optparse
import traceback

#
# Запрос выгрузки продаж и помещения в очередь
#

pluginPath = 'plugins'
# Запрашиваемые параметры выгрузки
# если вызвали без параметров, то начало выгрузки = минус 2 часа от текущего времени, конец выгрузки = текущее время
CSReguestBegin = time.strftime("%Y-%m-%dT%H:%M:%S.0", time.localtime())
beginTime = int(CSReguestBegin[11:13])-2
if beginTime < 0:
    beginTime = beginTime + 24
    CSReguestBegin = CSReguestBegin[:11]+'%s' % beginTime+CSReguestBegin[13:]
elif beginTime < 10:
    CSReguestBegin = CSReguestBegin[:11]+'0%s' % beginTime+CSReguestBegin[13:] #'0%s' % beginTime
else:
    CSReguestBegin = CSReguestBegin[:11]+'%s' % beginTime+CSReguestBegin[13:]
print CSReguestBegin
CSReguestEnd = time.strftime("%Y-%m-%dT%H:%M:%S.0", time.localtime())
cashIdMacro = '%(cashid)s'
defaultPathSales = os.path.normpath('%s/sales_for_cs/sales%s.json' % (tempfile.gettempdir(), cashIdMacro+'_%(timestamp)s'))
format = 'artixinterchange'


def main(broker, cashCode, storeID, mode, dates = None, shifts = None, closedShifts = 0, saleStateId = None, unloadingShopId=None, unloadingCashCode=None):
    rez = status.errorUnload # результат. если запись в очередь пройдет нормально, то вернется 0
    messEn = messRu = ''
    try:
        loggerSync.info('Создаем соединение.')
        connection = Connection(broker)
        loggerSync.info('Открываем соединение.')
        connection.open()
    except Exception, e:
        messRu = 'Ошибка подключения к локальному сервису qpid. %s' % str(e)
        loggerSync.error(messRu)
        return status.errorProgramm, 'Failed to connect to the local service qpid. %s' % str(e), messRu
    try:
        loggerSync.info('Создаем сессию.')
        session = connection.session()
        loggerSync.info('Создаем отправителя сообщений.')
        queue = 'artix.backward.cs'
        if mode == 'server':
            queue = 'artix.backward.mcs'
        sender = session.sender('%s; {create: always, node:{ type: queue, durable: True}}' % queue, durable=True)
    
        # Вызываем скрипт выгрузки продаж
        try:
            if format != None:
                try:
                    plugin = __import__('%s.%s' % (pluginPath, format), fromlist='q')
                    # выполняем скрипт, соответствующий типу сообщения
                    loggerSync.info('Вызываем плагин %s для выгрузки продаж' % format.encode('utf8'))
                    status.updateSaleState(saleStateId, 'Call plugin unloading sales', 'Вызываем плагин выгрузки продаж', status.STATUS_DURING)
                    pathSales = os.path.normpath(defaultPathSales)
                    if not os.path.exists(os.path.dirname(pathSales)):
                        os.makedirs(os.path.dirname(pathSales))
                    rez, messEn, messRu = plugin.unload(pathSales, mode, dates = dates, shifts = shifts, closedShifts = closedShifts, 
                                                        saleStateId = saleStateId, unloadingShopId=unloadingShopId, unloadingCashCode=unloadingCashCode)
                    if rez == status.OK:
                        for root, dirs, files in os.walk(os.path.dirname(pathSales)):
                            if files:
                                for file in files:
                                    saleFile = os.path.join(root, file)
                                    # архивируем выгруженые продажи
                                    loggerSync.info('Архивируем выгруженые продажи.')
                                    status.updateSaleState(saleStateId, 'Archiving sales', 'Архивируем выгруженые продажи', status.STATUS_DURING)
                                    zipfile.ZipFile(saleFile+'.zip', 'w', zipfile.ZIP_DEFLATED).write(saleFile)
                                    f = open(saleFile+'.zip', 'rb')
                                    cashId = cashCode
                                    if mode == 'server':
                                        m = re.match(".*sales(.+)_.*", file)
                                        if m and m.group(1) != cashIdMacro:
                                            cashId = m.group(1)
                                    if dates:
                                        CSReguestBegin = dates.split('@')[0]
                                        CSReguestEnd = dates.split('@')[1]
                                        prop = {'cashCode': cashId, 'compressMethod':'zip', 'CSReguestBegin': CSReguestBegin, 'CSReguestEnd':CSReguestEnd, 'cashUnloadState':saleStateId}
                                    elif shifts:
                                        CSReguestBeginShift = shifts.split('@')[0]
                                        CSReguestEndShift = shifts.split('@')[1]
                                        prop = {'cashCode': cashId, 'compressMethod':'zip', 'CSReguestBeginShift': CSReguestBeginShift, 'CSReguestEndShift':CSReguestEndShift, 'cashUnloadState':saleStateId}
                                    loggerSync.info('Отправляем сообщение с продажами в очередь.')
                                    status.updateSaleState(saleStateId, 'Sending sales to queue', 'Отправляем сообщение с продажами в очередь', status.STATUS_DURING)
                                    sender.send(Message(content=f.read(), id=hex(hash(datetime.now())), properties=prop, durable=True))
                                    f.close()
                                    backupFile.archive(saleFile, 'sales')
                                    os.remove(saleFile+'.zip')
                                    messEn = 'Message with sales sent'
                                    messRu = "Сообщение с продажами отправлено"
                            else:
                                messEn = 'Not found sales for the requested period.'
                                messRu = "Не найдены продажи за запрошенный период."
                        rez = status.OK
                    else:
                        sender.close()
                        connection.close()
                        return rez, messEn, messRu
                except Exception, e:
                    messEn = 'Error while unloading sales. %s' % str(e)
                    messRu = "Ошибка при выгрузке. %s" % str(e)
                    loggerSync.error(messRu)
                    loggerSync.error(traceback.format_exc())
            else:
                messEn = 'Unknown unloading format.'
                messRu = "Формат выгрузки не указан."
                loggerSync.error(messRu)
        except ImportError, e:
            messEn = 'Unknown format unloading.'
            messRu = "Указан неизвестный формат выгрузки." + str(e)
            loggerSync.error(messRu)
    except Exception, e:
        rez = status.errorProgramm
        messEn, messRu = str(e)
        loggerSync.error(messRu)
        loggerSync.error(traceback.format_exc())
    sender.close()
    connection.close()
    loggerSync.info(rez)
    loggerSync.info('-----------------------')
    return rez, messEn, messRu

if __name__ == '__main__':
    import propertyReader
    
    parser = optparse.OptionParser()
    parser.add_option("-s", "--shifts", dest="shifts", help="@-separated shift range")
    parser.add_option("-d", "--dates", dest="dates", help="@-separated dates range. Date format: YY-mm-ddTHH:MM:SS.0")
    parser.add_option("-c", "--closedShifts", dest="closedShifts", default=0, type='int', 
                      help="Unload only closed, opened or closed and opened shifts. Use: 0-closed, 1-opened, 2-closedAndOpened  [default: %default]")
    parser.add_option("-K", "--cashCode", dest="cashCode", help="Unload for this cash (Parameter only for server mode)")
    parser.add_option("-S", "--shopCode", dest="shopCode", help="Unload for this shop (Parameter only for server mode)")
    (options, args) = parser.parse_args(sys.argv[1:])
    messEn = 'Start unload'
    messRu = 'Начинаем выгрузку продаж'
    try:
        if options.dates:
            fromDate = options.dates.split('@')[0]
            toDate = options.dates.split('@')[1]
            saleStateId = status.addSaleState(messEn, messRu, status.STATUS_DURING, fromDate=fromDate, toDate=toDate, callFromCS = False)
            rez, messEn, messRu = main(propertyReader.localBroker, propertyReader.cashCode, propertyReader.storeID, propertyReader.mode, dates = options.dates, 
                                       closedShifts = options.closedShifts, saleStateId = saleStateId, unloadingShopId=options.shopCode, unloadingCashCode=options.cashCode)
        elif options.shifts:
            fromShiftNum = options.shifts.split('@')[0]
            toShiftNum = options.shifts.split('@')[1]
            saleStateId = status.addSaleState(messEn, messRu, status.STATUS_DURING, fromShiftNum = fromShiftNum, toShiftNum = toShiftNum, callFromCS = False)
            rez, messEn, messRu = main(propertyReader.localBroker, propertyReader.cashCode, propertyReader.storeID, propertyReader.mode, shifts = options.shifts, 
                                       closedShifts = options.closedShifts, saleStateId = saleStateId, unloadingShopId=options.shopCode, unloadingCashCode=options.cashCode)
        else:
            print "Время выгрузки продаж или диапазон смен не указаны. \nВызываем выгрузку продаж за последние 2 часа"
            saleStateId = status.addSaleState(messEn, messRu, status.STATUS_DURING, fromDate=CSReguestBegin, toDate=CSReguestEnd, callFromCS = False)
            rez, messEn, messRu = main(propertyReader.localBroker, propertyReader.cashCode, propertyReader.storeID, propertyReader.mode, 
                                       dates = '%s@%s' % (CSReguestBegin, CSReguestEnd), saleStateId = saleStateId, unloadingShopId=options.shopCode, unloadingCashCode=options.cashCode)
        status.updateSaleState(saleStateId, messEn, messRu, status.STATUS_READY if rez == status.OK else status.STATUS_ERROR)
    except Exception, e:
        status.updateSaleState(saleStateId, str(e), str(e), status.STATUS_ERROR)
